@@ -3,24 +3,22 @@
 #include <cstring>
 #include <iostream>
 
-using namespace std;
-
 class json_parser_handler : public orcus::json_handler
 {
 public:
     void object_key(std::string_view key, bool /*transient*/)
     {
-        cout << "object key: " << key << endl;
+        std::cout << "object key: " << key << std::endl;
     }
 
     void string(std::string_view val, bool /*transient*/)
     {
-        cout << "string: " << val << endl;
+        std::cout << "string: " << val << std::endl;
     }
 
     void number(double val)
     {
-        cout << "number: " << val << endl;
+        std::cout << "number: " << val << std::endl;
     }
 };
 
@@ -28,7 +26,7 @@ int main()
 {
     const char* test_code = "{\"key1\": [1,2,3,4,5], \"key2\": 12.3}";
 
-    cout << "JSON string: " << test_code << endl;
+    std::cout << "JSON string: " << test_code << std::endl;
 
     // Instantiate the parser with an own handler.
     json_parser_handler hdl;
