@@ -80,6 +80,10 @@ public:
 
     virtual void set_format(ss::row_t, ss::col_t, ss::row_t, ss::col_t, std::size_t) override {}
 
+    virtual void set_column_format(ss::col_t, ss::col_t, std::size_t) override {}
+
+    virtual void set_row_format(ss::col_t, std::size_t) override {}
+
     virtual void fill_down_cells(ss::row_t, ss::col_t, ss::row_t) override {}
 };
 
@@ -130,6 +134,10 @@ public:
 
     virtual void set_segment_italic(bool) override {}
 
+    virtual void set_segment_superscript(bool) override {}
+
+    virtual void set_segment_subscript(bool) override {}
+
     virtual void append_segment(std::string_view s) override
     {
         m_current_string += s;
@@ -190,10 +198,11 @@ public:
 int main()
 {
     std::filesystem::path input_dir = std::getenv("INPUTDIR");
+    auto filepath = input_dir / "multi-sheets.ods";
 
     my_import_factory factory;
     orcus::orcus_ods loader(&factory);
-    loader.read_file(input_dir / "multi-sheets.ods");
+    loader.read_file(filepath.native());
 
     return EXIT_SUCCESS;
 }
