@@ -17,13 +17,14 @@ namespace spreadsheet { namespace iface { class import_factory; }}
 class ORCUS_DLLPUBLIC orcus_parquet : public iface::import_filter
 {
 public:
+    orcus_parquet() = delete;
     orcus_parquet(const orcus_parquet&) = delete;
     orcus_parquet& operator=(const orcus_parquet&) = delete;
 
     orcus_parquet(spreadsheet::iface::import_factory* factory);
     ~orcus_parquet();
 
-    static bool detect(const unsigned char* blob, std::size_t size);
+    static bool detect(std::string_view strm);
 
     virtual void read_file(std::string_view filepath) override;
     virtual void read_file(std::u16string_view filepath) override;
