@@ -26,10 +26,11 @@ public:
     orcus_xls_xml(spreadsheet::iface::import_factory* factory);
     ~orcus_xls_xml();
 
+    orcus_xls_xml() = delete;
     orcus_xls_xml(const orcus_xls_xml&) = delete;
     orcus_xls_xml& operator= (const orcus_xls_xml&) = delete;
 
-    static bool detect(const unsigned char* blob, size_t size);
+    static bool detect(std::string_view strm);
 
     virtual void read_file(std::string_view filepath) override;
     virtual void read_file(std::u16string_view filepath) override;
