@@ -7,9 +7,7 @@
 
 #include <orcus/json_structure_tree.hpp>
 #include <orcus/json_parser.hpp>
-#include <orcus/global.hpp>
 #include <orcus/string_pool.hpp>
-#include "pstring.hpp"
 
 #include "json_structure_mapper.hpp"
 
@@ -65,7 +63,7 @@ struct structure_node
      */
     int32_t child_count = 0;
 
-    pstring name; //< value of a key for a object key node.
+    std::string_view name; //< value of a key for a object key node.
 
     /**
      * For a value node that is an immediate child of an array node, these
@@ -173,6 +171,8 @@ structure_tree::node_properties to_node_properties(const structure_node& sn)
     return np;
 }
 
+void empty_callback(std::any) {}
+
 } // anonymous namespace
 
 struct structure_tree::impl
@@ -182,6 +182,8 @@ struct structure_tree::impl
     parse_scopes_type m_stack;
     string_pool m_pool;
 
+    structure_tree::callback_handler_type m_cb_on_repeat = empty_callback;
+
     impl() : m_root(nullptr) {}
     ~impl() {}
 
@@ -204,10 +206,10 @@ struct structure_tree::impl
         push_stack(node_type::object);
     }
 
-    void object_key(const char* p, size_t len, bool transient)
+    void object_key(std::string_view key, bool transient)
     {
         structure_node node(node_type::object_key);
-        node.name = pstring(p, len);
+        node.name = key;
 
         if (transient)
             node.name = m_pool.intern(node.name).first;
@@ -235,7 +237,7 @@ struct structure_tree::impl
         push_value();
     }
 
-    void string(const char* /*p*/, size_t /*len*/, bool /*transient*/)
+    void string(std::string_view /*val*/, bool /*transient*/)
     {
         push_value();
     }
@@ -418,6 +420,9 @@ private:
             {
                 // current node does have a child of specified type.
                 bool repeat = is_node_repeatable(node);
+                if (repeat)
+                    m_cb_on_repeat(node.type);
+
                 structure_node& child = **it;
                 child.repeat = repeat;
                 m_stack.emplace_back(child);
@@ -665,9 +670,20 @@ std::string structure_tree::walker::build_row_group_path() const
 structure_tree::structure_tree() : mp_impl(std::make_unique<impl>()) {}
 structure_tree::~structure_tree() {}
 
+void structure_tree::set_callback(callback_type type, callback_handler_type callback)
+{
+    switch (type)
+    {
+        case callback_type::on_repeat_node:
+            mp_impl->m_cb_on_repeat = std::move(callback);
+            break;
+        case callback_type::unknown:;
+    }
+}
+
 void structure_tree::parse(std::string_view stream)
 {
-    json_parser<impl> parser(stream.data(), stream.size(), *mp_impl);
+    json_parser<impl> parser(stream, *mp_impl);
     parser.parse();
 }
 
