@@ -14,29 +14,26 @@
 #include <iostream>
 #include <sstream>
 
-using namespace std;
-
 namespace orcus {
 
 void print_opc_rel::operator() (const opc_rel_t& v) const
 {
-    using namespace std;
-    cout << v.rid << ": " << v.target << " (" << v.type << ")" << endl;
+    std::cout << v.rid << ": " << v.target << " (" << v.type << ")" << std::endl;
 }
 
-std::string resolve_file_path(const std::string& dir_path, const std::string& file_name)
+std::string resolve_file_path(std::string_view dir_path, std::string_view file_name)
 {
     if (dir_path.empty())
-        return file_name;
+        return std::string{file_name};
 
-    const char* p = &dir_path[0];
+    const char* p = dir_path.data();
     const char* p_end = p + dir_path.size();
 
     bool has_root = *p == '/';
     if (has_root)
         ++p;
 
-    vector<pstring> dir_stack;
+    std::vector<std::string_view> dir_stack;
     const char* p_head = nullptr;
     for (; p != p_end; ++p)
     {
@@ -44,15 +41,15 @@ std::string resolve_file_path(const std::string& dir_path, const std::string& fi
         {
             if (!p_head)
                 // invalid directory path.
-                return file_name;
+                return std::string{file_name};
 
             size_t len = p - p_head;
-            pstring dir(p_head, len);
+            std::string_view dir(p_head, len);
             if (dir == "..")
             {
                 if (dir_stack.empty())
                     // invalid directory path.
-                    return file_name;
+                    return std::string{file_name};
 
                 dir_stack.pop_back();
             }
@@ -72,16 +69,15 @@ std::string resolve_file_path(const std::string& dir_path, const std::string& fi
     if (p_head)
     {
         // directory path must end with '/'.  This one doesn't.
-        return file_name;
+        return std::string{file_name};
     }
 
-    ostringstream full_path;
+    std::ostringstream full_path;
     if (has_root)
         full_path << '/';
 
-    vector<pstring>::const_iterator it = dir_stack.begin(), it_end = dir_stack.end();
-    for (; it != it_end; ++it)
-        full_path << *it << '/';
+    for (auto dir : dir_stack)
+        full_path << dir << '/';
 
     full_path << file_name;
 
