@@ -23,7 +23,7 @@ class xml_context_base;
 /**
  * Function object to print relationship information.
  */
-struct print_opc_rel : ::std::unary_function<opc_rel_t, void>
+struct print_opc_rel
 {
     void operator() (const opc_rel_t& v) const;
 };
@@ -38,7 +38,7 @@ struct print_opc_rel : ::std::unary_function<opc_rel_t, void>
  *
  * @return full file path.
  */
-std::string resolve_file_path(const std::string& dir_path, const std::string& file_name);
+std::string resolve_file_path(std::string_view dir_path, std::string_view file_name);
 
 void init_ooxml_context(xml_context_base& cxt);
 
