@@ -531,12 +531,10 @@ orcus_xml::orcus_xml(xmlns_repository& ns_repo, spreadsheet::iface::import_facto
 
 orcus_xml::~orcus_xml() {}
 
-bool orcus_xml::detect(const unsigned char* blob, size_t size)
+bool orcus_xml::detect(std::string_view strm)
 {
-    std::string_view stream{reinterpret_cast<const char*>(blob), size};
-
     xml_detection_handler hdl;
-    sax_parser<xml_detection_handler> parser(stream, hdl);
+    sax_parser<xml_detection_handler> parser(strm, hdl);
 
     try
     {
@@ -641,6 +639,8 @@ void orcus_xml::read_stream(std::string_view stream)
 
     sax_ns_parser<xml_data_sax_handler> parser(stream, ns_cxt, handler);
     parser.parse();
+
+    mp_impl->im_factory->finalize();
 }
 
 #if ORCUS_DEBUG_XML
