@@ -14,8 +14,6 @@
 
 #include <iostream>
 
-using namespace std;
-
 namespace orcus {
 
 xlsx_drawing_context::xlsx_drawing_context(session_context& cxt, const tokens& tkns) :
@@ -27,11 +25,6 @@ xlsx_drawing_context::xlsx_drawing_context(session_context& cxt, const tokens& t
 
 xlsx_drawing_context::~xlsx_drawing_context() {}
 
-bool xlsx_drawing_context::can_handle_element(xmlns_id_t /*ns*/, xml_token_t /*name*/) const
-{
-    return true;
-}
-
 xml_context_base* xlsx_drawing_context::create_child_context(xmlns_id_t /*ns*/, xml_token_t /*name*/)
 {
     return nullptr;
@@ -41,7 +34,7 @@ void xlsx_drawing_context::end_child_context(xmlns_id_t /*ns*/, xml_token_t /*na
 {
 }
 
-void xlsx_drawing_context::start_element(xmlns_id_t ns, xml_token_t name, const::std::vector<xml_token_attr_t>& attrs)
+void xlsx_drawing_context::start_element(xmlns_id_t ns, xml_token_t name, const::std::vector<xml_token_attr_t>& /*attrs*/)
 {
     xml_token_pair_t parent = push_stack(ns, name);
 
@@ -127,7 +120,11 @@ bool xlsx_drawing_context::end_element(xmlns_id_t ns, xml_token_t name)
             case XML_twoCellAnchor:
             case XML_oneCellAnchor:
                 if (get_config().debug)
-                    cout << "col: " << m_col << "; row: " << m_row << "; col offset: " << m_col_offset << "; row offset: " << m_row_offset << endl;
+                {
+                    std::cout << "col: " << m_col << "; row: " << m_row
+                        << "; col offset: " << m_col_offset << "; row offset: "
+                        << m_row_offset << std::endl;
+                }
                 break;
             default:
                 ;
@@ -140,9 +137,9 @@ bool xlsx_drawing_context::end_element(xmlns_id_t ns, xml_token_t name)
     return pop_stack(ns, name);
 }
 
-void xlsx_drawing_context::characters(const pstring& str, bool /*transient*/)
+void xlsx_drawing_context::characters(std::string_view str, bool /*transient*/)
 {
-    const xml_token_pair_t& elem = get_current_element();
+    xml_token_pair_t elem = get_current_element();
     if (elem.first == NS_ooxml_xdr)
     {
         switch (elem.second)
