@@ -6,7 +6,6 @@
  */
 
 #include "xlsx_handler.hpp"
-#include "xlsx_context.hpp"
 #include "xlsx_sheet_context.hpp"
 #include "xlsx_table_context.hpp"
 #include "xlsx_pivot_context.hpp"
@@ -14,16 +13,16 @@
 
 #include <iostream>
 
-using namespace std;
+namespace ss = orcus::spreadsheet;
 
 namespace orcus {
 
 xlsx_sheet_xml_handler::xlsx_sheet_xml_handler(
-    session_context& session_cxt, const tokens& tokens,
+    session_context& session_cxt, const tokens& t,
     spreadsheet::sheet_t sheet_id,
     spreadsheet::iface::import_reference_resolver& resolver,
     spreadsheet::iface::import_sheet& sheet) :
-    xml_stream_handler(new xlsx_sheet_context(session_cxt, tokens, sheet_id, resolver, sheet))
+    xml_stream_handler(session_cxt, t, std::make_unique<xlsx_sheet_context>(session_cxt, t, sheet_id, resolver, sheet))
 {
 }
 
@@ -38,18 +37,18 @@ void xlsx_sheet_xml_handler::pop_rel_extras(opc_rel_extras_t& other)
 }
 
 xlsx_table_xml_handler::xlsx_table_xml_handler(
-    session_context& session_cxt, const tokens& tokens,
+    session_context& session_cxt, const tokens& t,
     spreadsheet::iface::import_table& table,
     spreadsheet::iface::import_reference_resolver& resolver) :
-    xml_stream_handler(new xlsx_table_context(session_cxt, tokens, table, resolver))
+    xml_stream_handler(session_cxt, t, std::make_unique<xlsx_table_context>(session_cxt, t, table, resolver))
 {
 }
 
 xlsx_pivot_cache_def_xml_handler::xlsx_pivot_cache_def_xml_handler(
-    session_context& cxt, const tokens& tokens,
+    session_context& cxt, const tokens& t,
     spreadsheet::iface::import_pivot_cache_definition& pcache,
     spreadsheet::pivot_cache_id_t pcache_id) :
-    xml_stream_handler(new xlsx_pivot_cache_def_context(cxt, tokens, pcache, pcache_id)) {}
+    xml_stream_handler(cxt, t, std::make_unique<xlsx_pivot_cache_def_context>(cxt, t, pcache, pcache_id)) {}
 
 opc_rel_extras_t xlsx_pivot_cache_def_xml_handler::pop_rel_extras()
 {
@@ -60,17 +59,19 @@ opc_rel_extras_t xlsx_pivot_cache_def_xml_handler::pop_rel_extras()
 }
 
 xlsx_pivot_cache_rec_xml_handler::xlsx_pivot_cache_rec_xml_handler(
-    session_context& cxt, const tokens& tokens,
+    session_context& cxt, const tokens& t,
     spreadsheet::iface::import_pivot_cache_records& pc_records) :
-    xml_stream_handler(new xlsx_pivot_cache_rec_context(cxt, tokens, pc_records)) {}
+    xml_stream_handler(cxt, t, std::make_unique<xlsx_pivot_cache_rec_context>(cxt, t, pc_records)) {}
 
 xlsx_pivot_table_xml_handler::xlsx_pivot_table_xml_handler(
-    session_context& cxt, const tokens& tokens) :
-    xml_stream_handler(new xlsx_pivot_table_context(cxt, tokens)) {}
+    session_context& cxt, const tokens& t,
+    ss::iface::import_pivot_table_definition& xpt,
+    ss::iface::import_reference_resolver& resolver) :
+    xml_stream_handler(cxt, t, std::make_unique<xlsx_pivot_table_context>(cxt, t, xpt, resolver)) {}
 
 xlsx_drawing_xml_handler::xlsx_drawing_xml_handler(
-    session_context& cxt, const tokens& tokens) :
-    xml_stream_handler(new xlsx_drawing_context(cxt, tokens)) {}
+    session_context& cxt, const tokens& t) :
+    xml_stream_handler(cxt, t, std::make_unique<xlsx_drawing_context>(cxt, t)) {}
 
 }
 
