@@ -15,14 +15,18 @@
 
 using namespace orcus;
 
-int main(int argc, char** argv) try
+int ORCUS_CLI_MAIN(int argc, arg_char_t** argv) try
 {
+    bootstrap_program();
+
     spreadsheet::range_size_t ss{1048576, 16384};
     spreadsheet::document doc{ss};
     spreadsheet::import_factory fact(doc);
     orcus_gnumeric app(&fact);
 
-    if (!parse_import_filter_args(argc, argv, fact, app, doc))
+    import_filter_arg_parser<arg_char_t> parser(fact, app, doc);
+
+    if (!parser.parse(argc, argv))
         return EXIT_FAILURE;
 
     return EXIT_SUCCESS;
