@@ -12,7 +12,6 @@
 #include "orcus/spreadsheet/factory.hpp"
 #include "orcus/spreadsheet/document.hpp"
 #include "orcus/stream.hpp"
-#include "orcus/global.hpp"
 #include "orcus/sax_parser_base.hpp"
 
 #include "orcus_filter_global.hpp"
@@ -23,13 +22,12 @@
 #include <iostream>
 #include <fstream>
 #include <boost/program_options.hpp>
-#include <boost/filesystem.hpp>
 #include <mdds/sorted_string_map.hpp>
 
+#include "filesystem_env.hpp"
+
 using namespace orcus;
-using namespace std;
 namespace po = boost::program_options;
-namespace fs = boost::filesystem;
 
 namespace {
 
@@ -44,21 +42,20 @@ enum class type {
     structure,
 };
 
-typedef mdds::sorted_string_map<type> map_type;
+using map_type = mdds::sorted_string_map<type>;
 
 // Keys must be sorted.
-const std::vector<map_type::entry> entries =
-{
-    { ORCUS_ASCII("dump"),      type::dump          },
-    { ORCUS_ASCII("map"),       type::map           },
-    { ORCUS_ASCII("map-gen"),   type::map_gen       },
-    { ORCUS_ASCII("structure"), type::structure     },
-    { ORCUS_ASCII("transform"), type::transform_xml },
+constexpr map_type::entry_type entries[] = {
+    { "dump",      type::dump          },
+    { "map",       type::map           },
+    { "map-gen",   type::map_gen       },
+    { "structure", type::structure     },
+    { "transform", type::transform_xml },
 };
 
 const map_type& get()
 {
-    static map_type mt(entries.data(), entries.size(), type::unknown);
+    static const map_type mt(entries, std::size(entries), type::unknown);
     return mt;
 }
 
@@ -66,16 +63,16 @@ const map_type& get()
 
 std::string to_string(output_mode::type t)
 {
-    for (const output_mode::map_type::entry& e : output_mode::entries)
+    for (const output_mode::map_type::entry_type& e : output_mode::entries)
         if (t == e.value)
-            return std::string(e.key, e.keylen);
+            return std::string(e.key);
 
     return std::string();
 }
 
-void print_usage(ostream& os, const po::options_description& desc)
+void print_usage(std::ostream& os, const po::options_description& desc)
 {
-    os << "Usage: orcus-xml [OPTIONS] FILE" << endl << endl;
+    os << "Usage: orcus-xml [OPTIONS] FILE" << std::endl << std::endl;
     os << desc;
 }
 
@@ -90,13 +87,13 @@ std::string build_mode_help_text()
 {
     std::ostringstream os;
     os << "Mode of operation. Select one of the following options: ";
-    auto it = output_mode::entries.cbegin(), ite = output_mode::entries.cend();
+    auto it = output_mode::entries, ite = output_mode::entries + std::size(output_mode::entries);
     --ite;
 
     for (; it != ite; ++it)
-        os << std::string(it->key, it->keylen) << ", ";
+        os << std::string(it->key) << ", ";
 
-    os << "or " << std::string(it->key, it->keylen) << ".";
+    os << "or " << std::string(it->key) << ".";
     return os.str();
 }
 
@@ -117,14 +114,14 @@ bool parse_and_dump_structure(const file_content& content, const std::string& ou
 
     if (output.empty())
     {
-        tree.dump_compact(cout);
+        tree.dump_compact(std::cout);
         return true;
     }
 
-    ofstream file(output);
+    std::ofstream file(output);
     if (!file)
     {
-        cerr << "failed to create output file: " << output << endl;
+        std::cerr << "failed to create output file: " << output << std::endl;
         return false;
     }
 
@@ -153,12 +150,12 @@ int main(int argc, char** argv) try
         ("mode", po::value<std::string>(), build_mode_help_text().data())
         ("map,m", po::value<std::string>(), build_map_help_text().data())
         ("output,o", po::value<std::string>(), build_output_help_text().data())
-        ("output-format,f", po::value<string>(), gen_help_output_format().data())
+        ("output-format,f", po::value<std::string>(), gen_help_output_format().data())
     ;
 
     po::options_description hidden("");
     hidden.add_options()
-        ("input", po::value<string>(), "input file");
+        ("input", po::value<std::string>(), "input file");
 
     po::positional_options_description po_desc;
     po_desc.add("input", 1);
@@ -173,41 +170,41 @@ int main(int argc, char** argv) try
             po::command_line_parser(argc, argv).options(cmd_opt).positional(po_desc).run(), vm);
         po::notify(vm);
     }
-    catch (const exception& e)
+    catch (const std::exception& e)
     {
         // Unknown options.
-        cerr << e.what() << endl;
-        print_usage(cout, desc);
+        std::cerr << e.what() << std::endl;
+        print_usage(std::cout, desc);
         return EXIT_FAILURE;
     }
 
     if (vm.count("help"))
     {
-        print_usage(cout, desc);
+        print_usage(std::cout, desc);
         return EXIT_FAILURE;
     }
 
     if (!vm.count("input"))
     {
-        cerr << "No input file." << endl;
-        print_usage(cout, desc);
+        std::cerr << "No input file." << std::endl;
+        print_usage(std::cout, desc);
         return EXIT_FAILURE;
     }
 
     if (!vm.count("mode"))
     {
-        cerr << "Mode not specified." << endl;
-        print_usage(cout, desc);
+        std::cerr << "Mode not specified." << std::endl;
+        print_usage(std::cout, desc);
         return EXIT_FAILURE;
     }
 
     std::string s = vm["mode"].as<std::string>();
-    output_mode::type mode = output_mode::get().find(s.data(), s.size());
+    output_mode::type mode = output_mode::get().find(s);
 
     if (mode == output_mode::type::unknown)
     {
-        cerr << "Unknown output mode: " << s << endl;
-        print_usage(cout, desc);
+        std::cerr << "Unknown output mode: " << s << std::endl;
+        print_usage(std::cout, desc);
         return EXIT_FAILURE;
     }
 
@@ -215,7 +212,7 @@ int main(int argc, char** argv) try
 
     if (!fs::is_regular_file(input_path))
     {
-        cerr << input_path << " is not a valid file." << endl;
+        std::cerr << input_path << " is not a valid file." << std::endl;
         return EXIT_FAILURE;
     }
 
@@ -226,7 +223,7 @@ int main(int argc, char** argv) try
 
         if (!fs::is_regular_file(map_path))
         {
-            cerr << map_path << " is not a valid map file." << endl;
+            std::cerr << map_path << " is not a valid map file." << std::endl;
             return EXIT_FAILURE;
         }
     }
@@ -296,14 +293,14 @@ int main(int argc, char** argv) try
                 }
                 else
                 {
-                    cerr << "Output format is not specified." << endl;
-                    print_usage(cout, desc);
+                    std::cerr << "Output format is not specified." << std::endl;
+                    print_usage(std::cout, desc);
                     return EXIT_FAILURE;
                 }
 
                 if (format == dump_format_t::unknown)
                 {
-                    std::cerr << "Unsupported output format: '" << s << "'" << endl;
+                    std::cerr << "Unsupported output format: '" << s << "'" << std::endl;
                     return EXIT_FAILURE;
                 }
 
@@ -314,15 +311,15 @@ int main(int argc, char** argv) try
             {
                 if (output.empty())
                 {
-                    cout << "output xml file name not provided" << endl;
-                    print_usage(cout, desc);
+                    std::cout << "output xml file name not provided" << std::endl;
+                    print_usage(std::cout, desc);
                     return EXIT_FAILURE;
                 }
 
-                ofstream file(output);
+                std::ofstream file(output);
                 if (!file)
                 {
-                    cerr << "failed to create output file: " << output << endl;
+                    std::cerr << "failed to create output file: " << output << std::endl;
                     return EXIT_FAILURE;
                 }
 
@@ -334,10 +331,10 @@ int main(int argc, char** argv) try
                 ;
         }
     }
-    catch (const sax::malformed_xml_error& e)
+    catch (const malformed_xml_error& e)
     {
-        cerr << create_parse_error_output(content.str(), e.offset()) << endl;
-        cerr << e.what() << endl;
+        std::cerr << create_parse_error_output(content.str(), e.offset()) << std::endl;
+        std::cerr << e.what() << std::endl;
         return EXIT_FAILURE;
     }
 
@@ -345,7 +342,7 @@ int main(int argc, char** argv) try
 }
 catch (const std::exception& e)
 {
-    cerr << e.what() << endl;
+    std::cerr << e.what() << std::endl;
     return EXIT_FAILURE;
 }
 
