@@ -13,11 +13,11 @@
 #include <orcus/spreadsheet/document.hpp>
 #include <orcus/spreadsheet/view.hpp>
 #include <orcus/exception.hpp>
-#include <orcus/global.hpp>
 #include <orcus/string_pool.hpp>
-#include "pstring.hpp"
 
 #include "factory_pivot.hpp"
+#include "factory_pivot_table_def.hpp"
+#include "factory_shared_strings.hpp"
 #include "factory_sheet.hpp"
 #include "global_settings.hpp"
 
@@ -166,10 +166,17 @@ public:
 
 using sheet_ifaces_type = std::vector<std::unique_ptr<import_sheet>>;
 
-}
+} // anonymous namespace
+
+import_factory_config::import_factory_config() = default;
+import_factory_config::import_factory_config(const import_factory_config& other) = default;
+import_factory_config::~import_factory_config() = default;
+
+import_factory_config& import_factory_config::operator=(const import_factory_config& other) = default;
 
 struct import_factory::impl
 {
+    std::shared_ptr<import_factory_config> m_config;
     import_factory& m_envelope;
     document& m_doc;
     view* m_view;
@@ -178,9 +185,11 @@ struct import_factory::impl
     import_global_settings m_global_settings;
     import_pivot_cache_def m_pc_def;
     import_pivot_cache_records m_pc_records;
+    detail::import_pivot_table_def pivot_table_def;
     import_ref_resolver m_ref_resolver;
     import_global_named_exp m_global_named_exp;
     import_styles m_styles;
+    detail::import_shared_strings shared_strings;
 
     sheet_ifaces_type m_sheets;
 
@@ -188,6 +197,7 @@ struct import_factory::impl
     formula_error_policy_t m_error_policy;
 
     impl(import_factory& envelope, document& doc) :
+        m_config(std::make_shared<import_factory_config>()),
         m_envelope(envelope),
         m_doc(doc),
         m_view(nullptr),
@@ -195,21 +205,25 @@ struct import_factory::impl
         m_global_settings(envelope, doc),
         m_pc_def(doc),
         m_pc_records(doc),
+        pivot_table_def(doc),
         m_ref_resolver(doc),
         m_global_named_exp(doc),
-        m_styles(doc.get_styles(), doc.get_string_pool()),
+        m_styles(m_config, doc.get_styles(), doc.get_string_pool()),
+        shared_strings(doc.get_string_pool(), doc.get_model_context(), doc.get_styles(), doc.get_shared_strings()),
         m_recalc_formula_cells(false),
-        m_error_policy(formula_error_policy_t::fail) {}
+        m_error_policy(formula_error_policy_t::fail)
+    {
+    }
 };
 
 import_factory::import_factory(document& doc) :
     mp_impl(std::make_unique<impl>(*this, doc)) {}
 
-import_factory::import_factory(document& doc, view& view) :
+import_factory::import_factory(document& doc, view& view_store) :
     mp_impl(std::make_unique<impl>(*this, doc))
 {
     // Store the optional view store.
-    mp_impl->m_view = &view;
+    mp_impl->m_view = &view_store;
 }
 
 import_factory::~import_factory() {}
@@ -221,7 +235,7 @@ iface::import_global_settings* import_factory::get_global_settings()
 
 iface::import_shared_strings* import_factory::get_shared_strings()
 {
-    return mp_impl->m_doc.get_shared_strings();
+    return &mp_impl->shared_strings;
 }
 
 iface::import_styles* import_factory::get_styles()
@@ -259,6 +273,12 @@ iface::import_pivot_cache_records* import_factory::create_pivot_cache_records(
     return &mp_impl->m_pc_records;
 }
 
+iface::import_pivot_table_definition* import_factory::create_pivot_table_definition()
+{
+    mp_impl->pivot_table_def.reset();
+    return &mp_impl->pivot_table_def;
+}
+
 iface::import_sheet* import_factory::append_sheet(sheet_t sheet_index, std::string_view name)
 {
     assert(sheet_index == static_cast<sheet_t>(mp_impl->m_doc.get_sheet_count()));
@@ -301,12 +321,18 @@ iface::import_sheet* import_factory::get_sheet(sheet_t sheet_index)
 
 void import_factory::finalize()
 {
-    mp_impl->m_doc.finalize();
+    mp_impl->m_doc.finalize_import();
 
     if (mp_impl->m_recalc_formula_cells)
         mp_impl->m_doc.recalc_formula_cells();
 }
 
+void import_factory::set_config(const import_factory_config& config)
+{
+    // NB: update the object state.
+    *mp_impl->m_config = config;
+}
+
 void import_factory::set_default_row_size(row_t row_size)
 {
     range_size_t ss = mp_impl->m_doc.get_sheet_size();
@@ -381,7 +407,7 @@ struct export_factory::impl
 export_factory::export_factory(const document& doc) :
     mp_impl(std::make_unique<impl>(doc)) {}
 
-export_factory::~export_factory() {}
+export_factory::~export_factory() = default;
 
 const iface::export_sheet* export_factory::get_sheet(std::string_view sheet_name) const
 {
