@@ -7,7 +7,6 @@
 
 #include "factory_pivot.hpp"
 
-#include "orcus/global.hpp"
 #include "orcus/string_pool.hpp"
 #include "orcus/exception.hpp"
 
@@ -172,7 +171,7 @@ void import_pivot_cache_def::set_field_name(std::string_view name)
     m_current_field.name = intern(name);
 }
 
-iface::import_pivot_cache_field_group* import_pivot_cache_def::create_field_group(size_t base_index)
+iface::import_pivot_cache_field_group* import_pivot_cache_def::start_field_group(size_t base_index)
 {
     m_current_field_group =
         std::make_unique<import_pc_field_group>(m_doc, m_current_field, base_index);
@@ -203,6 +202,7 @@ void import_pivot_cache_def::set_field_max_date(const date_time_t& dt)
 void import_pivot_cache_def::commit_field()
 {
     m_current_fields.push_back(std::move(m_current_field));
+    m_current_field = pivot_cache_field_t();
 }
 
 void import_pivot_cache_def::set_field_item_string(std::string_view value)
@@ -232,6 +232,7 @@ void import_pivot_cache_def::set_field_item_error(error_value_t ev)
 void import_pivot_cache_def::commit_field_item()
 {
     m_current_field.items.push_back(std::move(m_current_field_item));
+    m_current_field_item = pivot_cache_item_t();
 }
 
 void import_pivot_cache_def::commit()
