@@ -8,7 +8,6 @@
 #include "global_settings.hpp"
 #include "orcus/spreadsheet/document.hpp"
 #include "orcus/spreadsheet/factory.hpp"
-#include "orcus/global.hpp"
 
 namespace orcus { namespace spreadsheet {
 
@@ -22,7 +21,7 @@ struct import_global_settings::impl
 };
 
 import_global_settings::import_global_settings(import_factory& factory, document& doc) :
-    mp_impl(orcus::make_unique<impl>(factory, doc)) {}
+    mp_impl(std::make_unique<impl>(factory, doc)) {}
 
 import_global_settings::~import_global_settings() {}
 
